import { Op } from "sequelize";
import { ERRORENUM } from "../../config/errorEnum";
import { modelMap } from "../../model/modelBind";
import { BizError } from "../../util/bizError";

function analysisParamToWhere(param, column) {
    let where
    for (let key in param) {// 模糊查询{"字段名":{"%like%":'三'}}
        if (param[key]["%like%"]) {
            where[key] = {[Op.like]:`%${param[key]["%like%"]}%`};
        } else {
            where[key] = param[key];
        }
    }

    let selectParam:any = {where};
    if (column.length) selectParam.attributes = column;
    return selectParam
}

/**
 * 查询单个数据
 * @param tableModel 表对象
 * @param param 
 * @returns 
 */
export async function selectOneDataByParam(tableModel, param, column) {
    let selectParam = analysisParamToWhere(param, column);
    let data = tableModel.findOne(selectParam);
    return {data};
}


/**
 * 查询多个数据
 * @param tableName 表对象
 * @param param 
 * @returns 
 */
export async function selectDataListByParam(tableModel, param, column) {
    let selectParam = analysisParamToWhere(param, column);
    let data = tableModel.findAll(selectParam);
    return {data};
}


/**
 * 分页查询
 * @param tableModel 
 * @param param 
 * @param pageNumber 
 * @param pageSize 
 * @returns 
 */
export async function selectDataListToPageByParam(tableModel, param, column, pageNumber:number, pageSize:number) {
    let selectParam:any = analysisParamToWhere(param, column);
    selectParam.limit = pageSize || 10;
    selectParam.offset = (pageNumber-1)*10;
    let data = tableModel.findAll(selectParam);
    return {data};
}

export async function selectDataCountByParam(tableModel, param) {
    let selectParam:any = analysisParamToWhere(param, []);
    let data = tableModel.count(selectParam);
    return {data};
}


export async function associationSelect(tableName:string, param) {
    let model = modelMap[tableName];
    if (!model) throw new BizError(ERRORENUM.不存在表);

    let data = await model.aggragateData(param);

    return {data};
}


/**
 * 多表联查
 * @param tableModel 
 * @param includeConf {"表名":["",""] } 
 * @param param 
 * @param column 
 * @returns 
 */
export async function selectDataToTableAssociation(tableModel, includeConf, param, column) {
    let include = [];
    for (let tableName in includeConf) {
        if (!modelMap[tableName]) throw new BizError(ERRORENUM.不存在表, `尝试进行多表联查，但是不存在${tableName}`);
        include.push({
            attributes : includeConf[tableName],
            module : modelMap[tableName]
        });
    }

    let selectParam:any = analysisParamToWhere(param, column);
    selectParam.include = include;

    let data = tableModel.findAll(selectParam);
    return {data};
}


/**
 * 多表联查
 * @param tableModel 
 * @param includeConf {"表名":["",""] } 
 * @param param 
 * @param column 
 * @returns 
 */
export async function selectDataToTableAssociationToPage(tableModel, includeConf, param, column, pageNumber:number, pageSize:number) {
    let include = [];
    for (let tableName in includeConf) {
        if (!modelMap[tableName]) throw new BizError(ERRORENUM.不存在表, `尝试进行多表联查，但是不存在${tableName}`);
        include.push({
            attributes : includeConf[tableName],
            module : modelMap[tableName]
        });
    }

    let selectParam:any = analysisParamToWhere(param, column);
    selectParam.include = include;
    selectParam.limit = pageSize || 10;
    selectParam.offset = (pageNumber-1)*10;

    let data = tableModel.findAll(selectParam);
    return {data};
}